/* adc Scicos Analog to Digital Converter block
 * Type 4 simulation function ver 1.0 - scilab-3.0
 * 3 Mars 2006 - INRIA - Author : A.Layec
 */

/* REVISION HISTORY :
 * $Log$
 */

#include <math.h>
#include <stdio.h>
#include "scicos_block.h"

/*based on f2c definition*/
#define d_nint(x) ((x) > 0 ? floor((x)+.5) : -floor(.5 - (x)))

/* Cette fonction de calcul ralise la convertion analogique numrique.
 * Les valeurs de sorties sont des valeurs entires signes ou non-signes.
 *
 * entre rgulire : u[0..nu-1] vecteur des entres rgulires  convertir
 * sortie rgulire : y1[0..ny-1] vecteur des sorties converties
 *                    y2[0..ny-1] vecteur des sorties de bruit de quantification
 *
 * entre vnementielle : Dates de dclenchement
 * sortie vnementeille : nant
 *
 * paramtre entier : ipar[0..nu-1]     : mthode de quantification
 *                                        0 - contre-marche
 *                                        1 - mi-marche
 *                   ipar[nu..2*nu-1]   : Nombre de bits de la valeur entire de sortie
 *                   ipar[2*nu..3*nu-1] : flag code complment  2
 *                                        0 : nombre entier non sign
 *                                        1 : nombre entier sign
 *                   ipar[3*nu]         : flag calcul bruit de quantification
 *                                        0 : pas de calcul
 *                                        1 : calcul sur sortie rgulire 2
 *
 * paramtre rel : rpar[0..nu-1]      : pas (ou quantum)
 *                  rpar[nu..2*nu-1]   : valeur minimale ou dbute la conversion
 *                  rpar[2*nu..3*nu-1] : erreur d'offset
 *                  rpar[3*nu..4*nu-1] : erreur fsr
 */

/*
 * adcv_c routine de calcul d'un convertisseur analogique numrique
 *
 * Entres :
 * n       : taille des vecteurs
 * type    : type de quantification (vecteur)
 *           0 - contre marche
 *           1 - mi-marche
 * nbit    : longueur en bit de la valeur dcimale de sortie (vecteur)
 * cc2     : flag code complment  2 (vecteur)
 *           0 : nombre entier non sign
 *           1 : nombre entier sign
 * qn      : flag calcul bruit de quantification (scalaire)
 *           0 : pas de calcul
 *           1 : calcul sur sortie rgulire 2
 * q       : pas de quantification (vecteur)
 * vmin    : valeur minimale du dbut de la quantification (vecteur)
 * of_er   : erreur d'offset (vecteur)
 * fsr_er  : erreur fsr (vecteur)
 * u       : vecteur d'entre (vecteur)
 *
 * Sorties :
 * y1      : vecteur de sortie du convertisseur (vecteur)
 * y2      : vecteur de sortie du bruit de quantification (vecteur)
 *
 * Dpendances :
 * math.h
 */
void adcv_c(int *n,int *type,int *nbit, int *cc2,int *qn,double *q,double *vmin,\
            double *of_er, double *fsr_er,double *u,double *y1,double *y2)
{
 /*Dclaration des variables compteurs*/
 int i;

 /*Dclaration de variables auxiliares*/
 double fsr; /*full scale range*/
 double fsr_dec; /*decalage du full scale range*/

 for(i=0;i<(*n);i++)
 {
   /*Calcul chelle maximale*/
   fsr=((double)((1<<nbit[i])-1))*q[i];

   /*Calcul erreur fsr*/
   if (fsr_er[i]!=0.0) fsr_dec=u[i]-u[i]*(vmin[i]+fsr+fsr_er[i])/(vmin[i]+fsr);
   else fsr_dec=0.0;

   /*Ralise quantification*/
   if (type[i]==0)
      /*Calcul pour une quantification  contre-marche*/
      y1[i]=q[i]*(d_nint((u[i]+of_er[i]+fsr_dec)/q[i]-.5));
   else if (type[i]==1)
      /*Calcul pour une quantification  mi-marche*/
      y1[i]=q[i]*(d_nint((u[i]+q[i]/2.0+of_er[i]+fsr_dec)/q[i]-.5));

   /*Test sur la grandeur de sortie(saturation)*/
   if(y1[i]<vmin[i]) y1[i]=vmin[i];
   else if(y1[i]>vmin[i]+fsr) y1[i]=vmin[i]+fsr;

   /*Calcul valeur finale*/
   y1[i]=(double)((int)((y1[i]-vmin[i])/q[i]));

   /*Calcul bruit de quantification*/
   if ((*qn)!=0) y2[i]=(y1[i]*q[i])-vmin[i]-u[i];

   /*Test code complment  2*/
   if(cc2[i]==1) y1[i]=y1[i]-((double)(1<<(nbit[i]-1)));
 }
 return;
}

/*prototype*/
void adc(scicos_block *block,int flag)
{
  /*Dclaration des variables*/
  double *u,*y1,*y2;

  int nu;

  int type;
  int nbit;
  int cc2;
  int qn;

  double q;
  double vmin;
  double of_er;
  double fsr_er;

  /*Rcupration des adresses des ports rguliers*/
  u=(double *)block->inptr[0];
  y1=(double *)block->outptr[0];

  /*Rcupration de la taille des ports*/
  nu=block->insz[0];

  /*Rcupre le type de quantification*/
  type=block->ipar[0];
  /*Rcupre le nombre de bit*/
  nbit=block->ipar[nu];
  /*Rcupre le type de code*/
  cc2=block->ipar[2*nu];
  /*Rcupre calcul de bruit de quantification*/
  qn=block->ipar[3*nu];

  /*Rcupre adresse sortie rgulire 2*/
  if (qn!=0) y2=(double *)block->outptr[1];

  /*Rcupre le quantum*/
  q=block->rpar[0];
  /*Rcupre la grandeur minimale d'entre*/
  vmin=block->rpar[nu];
  /*Rcupre l'erreur d'offset*/
  of_er=block->rpar[2*nu];
  /*Rcupre l'erreur d'offset*/
  fsr_er=block->rpar[3*nu];

  /*Appel routine adcv_c*/
  adcv_c(&nu,&type,&nbit,&cc2,&qn,&q,&vmin,&of_er,&fsr_er,&u[0],&y1[0],&y2[0]);
}
